@@ -42,13 +42,8 @@ robyn_converge <- function(OutputModels, n_cuts = 20, sd_qtref = 3, med_lowb = 2
   stopifnot(n_cuts > min(c(sd_qtref, med_lowb)) + 1)
 
   # Gather all trials
-  get_lists <- as.logical(grepl("trial", names(OutputModels)) * sapply(OutputModels, is.list))
-  OutModels <- OutputModels[get_lists]
-  for (i in seq_along(OutModels)) {
-    if (i == 1) df <- data.frame()
-    temp <- OutModels[[i]]$resultCollect$resultHypParam %>% mutate(trial = i)
-    df <- rbind(df, temp)
-  }
+  get_trials <- which(names(OutputModels) %in% paste0("trial", seq(OutputModels$trials)))
+  df <- bind_rows(lapply(OutputModels[get_trials], function(x) x$resultCollect$resultHypParam))
   calibrated <- isTRUE(sum(df$mape) > 0)
 
   # Calculate deciles
@@ -101,19 +96,20 @@ robyn_converge <- function(OutputModels, n_cuts = 20, sd_qtref = 3, med_lowb = 2
       mutate(median = signif(median, 2))
     last.qt <- tail(temp.df, 1)
     greater <- ">" # intToUtf8(8814)
-    temp <- glued(paste(
-      "{error_type} {did}converged: sd@qt.{quantile} {sd} {symb_sd} {sd_threh} &",
-      "|med@qt.{quantile}| {qtn_median} {symb_med} {med_threh}"
-    ),
-    error_type = last.qt$error_type,
-    did = ifelse(last.qt$flag_sd & last.qt$flag_med, "", "NOT "),
-    sd = signif(last.qt$last_sd, 2),
-    symb_sd = ifelse(last.qt$flag_sd, "<=", greater),
-    sd_threh = signif(last.qt$first_sd_avg, 2),
-    quantile = n_cuts,
-    qtn_median = signif(last.qt$last_med, 2),
-    symb_med = ifelse(last.qt$flag_med, "<=", greater),
-    med_threh = signif(last.qt$med_thres, 2)
+    temp <- glued(
+      paste(
+        "{error_type} {did}converged: sd@qt.{quantile} {sd} {symb_sd} {sd_threh} &",
+        "|med@qt.{quantile}| {qtn_median} {symb_med} {med_threh}"
+      ),
+      error_type = last.qt$error_type,
+      did = ifelse(last.qt$flag_sd & last.qt$flag_med, "", "NOT "),
+      sd = signif(last.qt$last_sd, 2),
+      symb_sd = ifelse(last.qt$flag_sd, "<=", greater),
+      sd_threh = signif(last.qt$first_sd_avg, 2),
+      quantile = n_cuts,
+      qtn_median = signif(last.qt$last_med, 2),
+      symb_med = ifelse(last.qt$flag_med, "<=", greater),
+      med_threh = signif(last.qt$med_thres, 2)
     )
     conv_msg <- c(conv_msg, temp)
   }
@@ -198,3 +194,37 @@ robyn_converge <- function(OutputModels, n_cuts = 20, sd_qtref = 3, med_lowb = 2
 
   return(invisible(cvg_out))
 }
+
+test_cvg <- function() {
+  # Experiment with gamma distribution fitting
+  gamma_mle <- function(params, x) {
+    gamma_shape <- params[[1]]
+    gamma_scale <- params[[2]]
+    # Negative log-likelihood
+    return(-sum(dgamma(x, shape = gamma_shape, scale = gamma_scale, log = TRUE)))
+  }
+  f_geo <- function(a, r, n) {
+    for (i in 2:n) a[i] <- a[i - 1] * r
+    return(a)
+  }
+  seq_nrmse <- f_geo(5, 0.7, 100)
+  df_nrmse <- data.frame(x = 1:100, y = seq_nrmse, type = "true")
+  mod_gamma <- nloptr(
+    x0 = c(1, 1), eval_f = gamma_mle, lb = c(0, 0),
+    x = seq_nrmse,
+    opts = list(algorithm = "NLOPT_LN_SBPLX", maxeval = 1e5)
+  )
+  gamma_params <- mod_gamma$solution
+  seq_nrmse_gam <- 1 / dgamma(seq_nrmse, shape = gamma_params[[1]], scale = gamma_params[[2]])
+  seq_nrmse_gam <- seq_nrmse_gam / (max(seq_nrmse_gam) - min(seq_nrmse_gam))
+  seq_nrmse_gam <- max(seq_nrmse) * seq_nrmse_gam
+  range(seq_nrmse_gam)
+  range(seq_nrmse)
+  df_nrmse_gam <- data.frame(x = 1:100, y = seq_nrmse_gam, type = "pred")
+  df_nrmse <- bind_rows(df_nrmse, df_nrmse_gam)
+  p <- ggplot(df_nrmse, aes(.data$x, .data$y, color = .data$type)) +
+    geom_line()
+  return(p)
+  # g_low = qgamma(0.025, shape=gamma_params[[1]], scale= gamma_params[[2]])
+  # g_up = qgamma(0.975, shape=gamma_params[[1]], scale= gamma_params[[2]])
+}
