@@ -15,14 +15,13 @@
 #' @param pareto_fronts Integer. Number of Pareto fronts for the output.
 #' \code{pareto_fronts = 1} returns the best models trading off \code{NRMSE} &
 #' \code{DECOMP.RSSD}. Increase \code{pareto_fronts} to get more model choices.
+#' \code{pareto_fronts = "auto"} selects the min fronts that include at least 100
+#' candidates. To customize this threshold, set value with \code{min_candidates}.
 #' @param calibration_constraint Numeric. Default to 0.1 and allows 0.01-0.1. When
 #' calibrating, 0.1 means top 10% calibrated models are used for pareto-optimal
 #' selection. Lower \code{calibration_constraint} increases calibration accuracy.
 #' @param plot_folder Character. Path for saving plots. Default
 #' to \code{robyn_object} and saves plot in the same directory as \code{robyn_object}.
-#' @param plot_folder_sub Character. Customize sub path to save plots. The total
-#' path is created with \code{dir.create(file.path(plot_folder, plot_folder_sub))}.
-#' For example, plot_folder_sub = "sub_dir".
 #' @param plot_pareto Boolean. Set to \code{FALSE} to deactivate plotting
 #' and saving model one-pagers. Used when testing models.
 #' @param clusters Boolean. Apply \code{robyn_clusters()} to output models?
@@ -34,30 +33,37 @@
 #' @param ui Boolean. Save additional outputs for UI usage. List outcome.
 #' @param export Boolean. Export outcomes into local files?
 #' @param quiet Boolean. Keep messages off?
+#' @param refresh Boolean. Refresh mode
 #' @param ... Additional parameters passed to \code{robyn_clusters()}
 #' @return (Invisible) list. Class: \code{robyn_outputs}. Contains processed
 #' results based on \code{robyn_run()} results.
 #' @export
 robyn_outputs <- function(InputCollect, OutputModels,
-                          pareto_fronts = 3,
+                          pareto_fronts = "auto",
                           calibration_constraint = 0.1,
-                          plot_folder = NULL, plot_folder_sub = NULL,
+                          plot_folder = NULL,
                           plot_pareto = TRUE,
                           csv_out = "pareto",
                           clusters = TRUE,
                           select_model = "clusters",
                           ui = FALSE, export = TRUE,
-                          quiet = FALSE, ...) {
+                          quiet = FALSE,
+                          refresh = FALSE, ...) {
   if (is.null(plot_folder)) plot_folder <- getwd()
-  plot_folder <- check_filedir(plot_folder)
+  plot_folder <- check_dir(plot_folder)
 
   # Check calibration constrains
-  calibration_constraint <- check_calibconstr(
-    calibration_constraint,
-    OutputModels$iterations,
-    OutputModels$trials,
-    InputCollect$calibration_input
-  )
+  calibrated <- !is.null(InputCollect$calibration_input)
+  all_fixed <- length(OutputModels$trial1$hyperBoundFixed) == length(OutputModels$hyper_updated)
+  if (!all_fixed) {
+    calibration_constraint <- check_calibconstr(
+      calibration_constraint,
+      OutputModels$iterations,
+      OutputModels$trials,
+      InputCollect$calibration_input,
+      refresh = refresh
+    )
+  }
 
   #####################################
   #### Run robyn_pareto on OutputModels
@@ -70,9 +76,24 @@ robyn_outputs <- function(InputCollect, OutputModels,
     ))
   }
   pareto_results <- robyn_pareto(
-    InputCollect, OutputModels, pareto_fronts, calibration_constraint, quiet
+    InputCollect, OutputModels,
+    pareto_fronts = pareto_fronts,
+    calibration_constraint = calibration_constraint,
+    quiet = quiet,
+    calibrated = calibrated,
+    ...
   )
-  allSolutions <- unique(pareto_results$xDecompVecCollect$solID)
+  pareto_fronts <- pareto_results$pareto_fronts
+  allSolutions <- pareto_results$pareto_solutions
+
+  # Reduce the size of xDecompVec with only pareto-front models and create solID
+  for (i in seq(OutputModels$trials)) {
+    OutputModels[names(OutputModels) %in% paste0("trial", i)][[1]]$resultCollect$xDecompVec <-
+      OutputModels[names(OutputModels) %in% paste0("trial", i)][[1]]$resultCollect$xDecompVec %>%
+      mutate(solID = paste(.data$trial, .data$iterNG, .data$iterPar, sep = "_")) %>%
+      filter(.data$solID %in% pareto_results$pareto_solutions) %>%
+      select(-c("iterNG", "iterPar"))
+  }
 
   #####################################
   #### Gather the results into output object
@@ -81,15 +102,35 @@ robyn_outputs <- function(InputCollect, OutputModels,
   allPareto <- list(
     resultHypParam = pareto_results$resultHypParam,
     xDecompAgg = pareto_results$xDecompAgg,
-    plotDataCollect = pareto_results$plotDataCollect
+    resultCalibration = pareto_results$resultCalibration,
+    plotDataCollect = pareto_results$plotDataCollect,
+    df_caov_pct = pareto_results$df_caov_pct_all
   )
 
+  # Set folder to save outputs: legacy plot_folder_sub
+  if (TRUE) {
+    depth <- ifelse(
+      "refreshDepth" %in% names(InputCollect),
+      InputCollect$refreshDepth,
+      ifelse("refreshCounter" %in% names(InputCollect),
+        InputCollect$refreshCounter, 0
+      )
+    )
+    folder_var <- ifelse(!as.integer(depth) > 0, "init", paste0("rf", depth))
+    plot_folder_sub <- paste("Robyn", format(Sys.time(), "%Y%m%d%H%M"), folder_var, sep = "_")
+  }
+
   # Final results object
   OutputCollect <- list(
     resultHypParam = filter(pareto_results$resultHypParam, .data$solID %in% allSolutions),
     xDecompAgg = filter(pareto_results$xDecompAgg, .data$solID %in% allSolutions),
     mediaVecCollect = pareto_results$mediaVecCollect,
     xDecompVecCollect = pareto_results$xDecompVecCollect,
+    resultCalibration = if (calibrated) {
+      filter(pareto_results$resultCalibration, .data$solID %in% allSolutions)
+    } else {
+      NULL
+    },
     allSolutions = allSolutions,
     allPareto = allPareto,
     calibration_constraint = calibration_constraint,
@@ -108,41 +149,69 @@ robyn_outputs <- function(InputCollect, OutputModels,
   )
   class(OutputCollect) <- c("robyn_outputs", class(OutputCollect))
 
-  # Set folder to save outputs
-  if (is.null(plot_folder_sub)) {
-    refresh <- attr(OutputModels, "refresh")
-    depth <- ifelse(
-      "refreshDepth" %in% names(InputCollect),
-      InputCollect$refreshDepth,
-      ifelse("refreshCounter" %in% names(InputCollect),
-        InputCollect$refreshCounter, 0
-      )
-    )
-    refresh <- as.integer(depth) > 0
-    folder_var <- ifelse(!refresh, "init", paste0("rf", depth))
-    plot_folder_sub <- paste("Robyn", format(Sys.time(), "%Y%m%d%H%M"), folder_var, sep = "_")
-  }
-
   plotPath <- paste0(plot_folder, "/", plot_folder_sub, "/")
   OutputCollect$plot_folder <- gsub("//", "/", plotPath)
+  if (export && !dir.exists(OutputCollect$plot_folder)) dir.create(OutputCollect$plot_folder, recursive = TRUE)
+
+  # Cluster results and amend cluster output
+  if (clusters) {
+    if (!quiet) message(">>> Calculating clusters for model selection using Pareto fronts...")
+    try(clusterCollect <- robyn_clusters(OutputCollect,
+      dep_var_type = InputCollect$dep_var_type,
+      quiet = quiet, export = export, ...
+    ))
+    OutputCollect$resultHypParam <- left_join(
+      OutputCollect$resultHypParam,
+      select(clusterCollect$data, .data$solID, .data$cluster, .data$top_sol),
+      by = "solID"
+    )
+    OutputCollect$xDecompAgg <- left_join(
+      OutputCollect$xDecompAgg,
+      select(clusterCollect$data, .data$solID, .data$cluster, .data$top_sol),
+      by = "solID"
+    ) %>%
+      left_join(
+        select(
+          clusterCollect$df_cluster_ci, .data$rn, .data$cluster, .data$boot_mean,
+          .data$boot_se, .data$ci_low, .data$ci_up, .data$rn
+        ),
+        by = c("rn", "cluster")
+      ) %>%
+      left_join(
+        pareto_results$df_caov_pct_all,
+        by = c("solID", "rn")
+      )
+    OutputCollect$mediaVecCollect <- left_join(
+      OutputCollect$mediaVecCollect,
+      select(clusterCollect$data, .data$solID, .data$cluster, .data$top_sol),
+      by = "solID"
+    )
+    OutputCollect$xDecompVecCollect <- left_join(
+      OutputCollect$xDecompVecCollect,
+      select(clusterCollect$data, .data$solID, .data$cluster, .data$top_sol),
+      by = "solID"
+    )
+    if (calibrated) {
+      OutputCollect$resultCalibration <- left_join(
+        OutputCollect$resultCalibration,
+        select(clusterCollect$data, .data$solID, .data$cluster, .data$top_sol),
+        by = "solID"
+      )
+    }
+    OutputCollect[["clusters"]] <- clusterCollect
+  }
 
   if (export) {
-    dir.create(OutputCollect$plot_folder)
     tryCatch(
       {
         if (!quiet) message(paste0(">>> Collecting ", length(allSolutions), " pareto-optimum results into: ", OutputCollect$plot_folder))
 
-        if (csv_out %in% c("all", "pareto")) {
-          if (!quiet) message(paste(">> Exporting", csv_out, "results as CSVs into directory..."))
-          robyn_csv(InputCollect, OutputCollect, csv_out, export = export)
-        }
-
         if (!quiet) message(">> Exporting general plots into directory...")
         all_plots <- robyn_plots(InputCollect, OutputCollect, export = export)
 
-        if (clusters) {
-          if (!quiet) message(">>> Calculating clusters for model selection using Pareto fronts...")
-          try(OutputCollect[["clusters"]] <- robyn_clusters(OutputCollect, quiet = quiet, export = export, ...))
+        if (csv_out %in% c("all", "pareto")) {
+          if (!quiet) message(paste(">> Exporting", csv_out, "results as CSVs into directory..."))
+          robyn_csv(InputCollect, OutputCollect, csv_out, export = export, calibrated = calibrated)
         }
 
         if (plot_pareto) {
@@ -151,7 +220,7 @@ robyn_outputs <- function(InputCollect, OutputModels,
               ">>> Exporting %sone-pagers into directory...", ifelse(!OutputCollect$hyper_fixed, "pareto ", "")
             ))
           }
-          select_model <- if (!clusters | is.null(OutputCollect[["clusters"]])) NULL else select_model
+          select_model <- if (!clusters || is.null(OutputCollect[["clusters"]])) NULL else select_model
           pareto_onepagers <- robyn_onepagers(
             InputCollect, OutputCollect,
             select_model = select_model,
@@ -163,7 +232,7 @@ robyn_outputs <- function(InputCollect, OutputModels,
         robyn_write(InputCollect, dir = OutputCollect$plot_folder, quiet = quiet)
 
         # For internal use -> UI Code
-        if (ui & plot_pareto) OutputCollect$UI$pareto_onepagers <- pareto_onepagers
+        if (ui && plot_pareto) OutputCollect$UI$pareto_onepagers <- pareto_onepagers
         OutputCollect[["UI"]] <- if (ui) list(pParFront = all_plots[["pParFront"]]) else NULL
       },
       error = function(err) {
@@ -206,20 +275,27 @@ Pareto-front ({x$pareto_fronts}) All solutions ({nSols}): {paste(x$allSolutions,
 #' Output results into local files: CSV files
 #'
 #' @param OutputCollect \code{robyn_run(..., export = FALSE)} output.
+#' @param calibrated Logical
 #' @rdname robyn_outputs
 #' @return Invisible \code{NULL}.
 #' @export
-robyn_csv <- function(InputCollect, OutputCollect, csv_out = NULL, export = TRUE) {
+robyn_csv <- function(InputCollect, OutputCollect, csv_out = NULL, export = TRUE, calibrated = FALSE) {
   if (export) {
     check_class("robyn_outputs", OutputCollect)
     temp_all <- OutputCollect$allPareto
     if ("pareto" %in% csv_out) {
       write.csv(OutputCollect$resultHypParam, paste0(OutputCollect$plot_folder, "pareto_hyperparameters.csv"))
       write.csv(OutputCollect$xDecompAgg, paste0(OutputCollect$plot_folder, "pareto_aggregated.csv"))
+      if (calibrated) {
+        write.csv(OutputCollect$resultCalibration, paste0(OutputCollect$plot_folder, "pareto_calibration.csv"))
+      }
     }
     if ("all" %in% csv_out) {
       write.csv(temp_all$resultHypParam, paste0(OutputCollect$plot_folder, "all_hyperparameters.csv"))
       write.csv(temp_all$xDecompAgg, paste0(OutputCollect$plot_folder, "all_aggregated.csv"))
+      if (calibrated) {
+        write.csv(temp_all$resultCalibration, paste0(OutputCollect$plot_folder, "all_calibration.csv"))
+      }
     }
     if (!is.null(csv_out)) {
       write.csv(InputCollect$dt_input, paste0(OutputCollect$plot_folder, "raw_data.csv"))
